@@ -17,7 +17,8 @@ import os
 import uvicorn
 import inspect
 
-from typing import Callable, override
+from typing import Callable
+from typing_extensions import override
 from a2a.server.agent_execution import AgentExecutor
 from a2a.server.agent_execution.context import RequestContext
 from a2a.server.apps import A2AStarletteApplication
