@@ -15,12 +15,16 @@
 """
 AgentKit Client Module
 
-Provides base client infrastructure for all AgentKit services.
+Provides base client infrastructure for all AgentKit and Volcengine services.
 """
 
-from agentkit.client.base_client import BaseAgentkitClient, ApiConfig
+from agentkit.client.base_service_client import BaseServiceClient, ApiConfig
+from agentkit.client.base_agentkit_client import BaseAgentkitClient
+from agentkit.client.base_iam_client import BaseIAMClient
 
 __all__ = [
-    'BaseAgentkitClient',
-    'ApiConfig',
+    "BaseServiceClient",
+    "BaseAgentkitClient",
+    "BaseIAMClient",
+    "ApiConfig",
 ]
