@@ -16,6 +16,7 @@ import os
 import logging
 import tempfile
 import uuid
+import sys
 from pathlib import Path
 from typing import Dict, Any, Optional
 from dataclasses import dataclass, field
@@ -624,6 +625,16 @@ class VeCPCRBuilder(Builder):
                     "build_script": docker_build_config.build_script,
                 }
 
+            from agentkit.toolkit.docker.dockerfile.metadata import (
+                calculate_template_hash,
+            )
+
+            template_path = Path(template_dir) / config.dockerfile_template
+            template_hash = calculate_template_hash(template_path)
+
+            config_hash_dict["dockerfile_template"] = config.dockerfile_template
+            config_hash_dict["dockerfile_template_hash"] = template_hash
+
             renderer = DockerfileRenderer(template_dir)
 
             # Content generator function (captures context via closure)
@@ -666,7 +677,8 @@ class VeCPCRBuilder(Builder):
         """
         try:
             from agentkit.toolkit.volcengine.utils.project_archiver import (
-                create_project_archive,
+                ArchiveConfig,
+                ProjectArchiver,
             )
 
             common_config = config.common_config
@@ -678,12 +690,54 @@ class VeCPCRBuilder(Builder):
             temp_dir = tempfile.mkdtemp()
             source_base_path = self.workdir
 
-            # Create archive using project archiver utility
-            archive_path = create_project_archive(
+            dockerignore_path = source_base_path / ".dockerignore"
+            dockerignore_path_str = (
+                str(dockerignore_path) if dockerignore_path.is_file() else None
+            )
+
+            archive_config = ArchiveConfig(
                 source_dir=str(source_base_path),
                 output_dir=temp_dir,
                 archive_name=archive_name,
+                dockerignore_path=dockerignore_path_str,
             )
+            archiver = ProjectArchiver(archive_config)
+            files_to_include = archiver.collect_files_to_include()
+
+            size_threshold_bytes = 100 * 1024 * 1024
+            total_size_bytes = 0
+            included_files_with_size: list[tuple[str, int]] = []
+            for p in files_to_include:
+                try:
+                    rel = p.relative_to(source_base_path).as_posix()
+                    size = p.stat().st_size
+                except Exception:
+                    continue
+                total_size_bytes += size
+                included_files_with_size.append((rel, size))
+
+            if total_size_bytes > size_threshold_bytes:
+                self._warn_large_archive(
+                    total_size_bytes=total_size_bytes,
+                    threshold_bytes=size_threshold_bytes,
+                    included_files_with_size=included_files_with_size,
+                )
+
+                if sys.stdin.isatty():
+                    confirmed = self.reporter.confirm(
+                        message=(
+                            "The archive to be uploaded is larger than 100 MiB. "
+                            "Do you want to continue uploading all included files?"
+                        ),
+                        default=False,
+                    )
+                    if not confirmed:
+                        raise Exception(
+                            "Archive upload cancelled by user (archive size exceeds 100 MiB)."
+                        )
+
+            # Create archive using pre-collected file list (avoid re-walking directories)
+            archive_path = archiver.create_archive(files_to_include=files_to_include)
 
             logger.info(f"Project archive created: {archive_path}")
             self.reporter.success(f"Project archive created: {archive_path}")
@@ -692,6 +746,91 @@ class VeCPCRBuilder(Builder):
         except Exception as e:
             raise Exception(f"Failed to create project archive: {str(e)}")
 
+    def _warn_large_archive(
+        self,
+        total_size_bytes: int,
+        threshold_bytes: int,
+        included_files_with_size: list[tuple[str, int]],
+    ) -> None:
+        def format_bytes(num_bytes: int) -> str:
+            units = ["B", "KiB", "MiB", "GiB", "TiB"]
+            value = float(num_bytes)
+            for unit in units:
+                if value < 1024 or unit == units[-1]:
+                    if unit == "B":
+                        return f"{int(value)} {unit}"
+                    return f"{value:.2f} {unit}"
+                value /= 1024
+            return f"{value:.2f} TiB"
+
+        included_count = len(included_files_with_size)
+        size_str = format_bytes(total_size_bytes)
+        threshold_str = format_bytes(threshold_bytes)
+
+        self.reporter.warning(
+            f"Archive size is {size_str} (threshold: {threshold_str}). Included files: {included_count}."
+        )
+
+        # Top-level folder summary (by total size)
+        top_level_sizes: dict[str, int] = {}
+        for rel_path, size in included_files_with_size:
+            top = rel_path.split("/", 1)[0] if "/" in rel_path else rel_path
+            key = f"{top}/" if top and top != rel_path else top
+            top_level_sizes[key] = top_level_sizes.get(key, 0) + size
+
+        top_level_lines = [
+            "Top-level paths by total size (descending):",
+        ]
+        for path, size in sorted(
+            top_level_sizes.items(), key=lambda kv: kv[1], reverse=True
+        )[:30]:
+            top_level_lines.append(f"  {format_bytes(size):>10}  {path}")
+
+        # Largest files list
+        largest_lines = ["Largest files (descending):"]
+        for rel_path, size in sorted(
+            included_files_with_size, key=lambda kv: kv[1], reverse=True
+        )[:50]:
+            largest_lines.append(f"  {format_bytes(size):>10}  {rel_path}")
+
+        # Sample file list (lexicographic)
+        sample_limit = 200
+        sample_lines = [f"Included files (first {sample_limit} of {included_count}):"]
+        for rel_path, _size in sorted(included_files_with_size, key=lambda kv: kv[0])[
+            :sample_limit
+        ]:
+            sample_lines.append(f"  {rel_path}")
+        if included_count > sample_limit:
+            sample_lines.append(
+                f"  ... ({included_count - sample_limit} more files not shown)"
+            )
+
+        hint_lines = [
+            "How to exclude files from the upload:",
+            "  1) Edit .dockerignore in your project root.",
+            "  2) Add ignore rules, for example:",
+            "     - data/",
+            "     - *.log",
+            "     - .venv/",
+            "     - **/*.bin",
+            "  3) Re-run agentkit build/launch.",
+        ]
+
+        lines = []
+        lines.extend(top_level_lines)
+        lines.append("")
+        lines.extend(largest_lines)
+        lines.append("")
+        lines.extend(sample_lines)
+        lines.append("")
+        lines.extend(hint_lines)
+
+        self.reporter.show_logs(
+            title="Large archive detected (review before upload)",
+            lines=lines,
+            max_lines=400,
+        )
+
     def _upload_to_tos(
         self, archive_path: str, config: VeCPCRBuilderConfig
     ) -> tuple[str, str]:
@@ -715,6 +854,7 @@ class VeCPCRBuilder(Builder):
             from agentkit.toolkit.volcengine.services.tos_service import (
                 TOSService,
                 TOSServiceConfig,
+                tos,
             )
 
             # Handle bucket configuration with auto-creation support
@@ -742,55 +882,95 @@ class VeCPCRBuilder(Builder):
 
             tos_service = TOSService(tos_config)
 
-            # Check if bucket exists and create if necessary
-            self.reporter.info(f"Checking TOS bucket existence: {bucket_name}")
-            if not tos_service.bucket_exists():
-                # Case 2: Bucket doesn't exist, create it
-                self.reporter.warning(
-                    f"TOS bucket does not exist, creating: {bucket_name}"
+            # Two-step safety check:
+            # 1) Ensure the bucket exists and is accessible.
+            # 2) Verify the bucket is owned by the current account via ListBuckets before uploading.
+            import time
+
+            created_in_this_run = False
+
+            # Step 1: ensure bucket exists / accessible
+            if auto_created_bucket:
+                self.reporter.info(
+                    f"Creating auto-generated TOS bucket in current account: {bucket_name}"
                 )
 
-                if not tos_service.create_bucket():
-                    error_msg = f"Failed to create TOS bucket: {bucket_name}"
-                    self.reporter.error(error_msg)
-                    logger.error(error_msg)
-                    raise Exception(error_msg)
-
-                self.reporter.success(f"TOS bucket created successfully: {bucket_name}")
-
-                # Verify bucket creation with retry logic (eventual consistency)
-                self.reporter.info(f"Verifying TOS bucket creation: {bucket_name}")
-                import time
-
-                start_time = time.time()
-                timeout = 30  # 30 second timeout
-                check_interval = 2  # Check every 2 seconds
-
-                while time.time() - start_time < timeout:
-                    if tos_service.bucket_exists():
-                        self.reporter.success(
-                            f"TOS bucket verification successful: {bucket_name}"
-                        )
+                # Very low probability: name collision. Retry with a new generated name.
+                max_attempts = 3
+                for attempt in range(1, max_attempts + 1):
+                    tos_service.config.bucket = bucket_name
+                    try:
+                        tos_service.create_bucket()
+                        created_in_this_run = True
                         break
-                    else:
-                        self.reporter.info(
-                            f"Waiting for TOS bucket to be ready... ({time.time() - start_time:.1f}s)"
-                        )
-                        time.sleep(check_interval)
-                else:
-                    # Timeout occurred
-                    error_msg = f"TOS bucket creation verification timeout ({timeout}s): {bucket_name}"
+                    except tos.exceptions.TosServerError as e:
+                        if e.status_code == 409 and attempt < max_attempts:
+                            bucket_name = TOSService.generate_bucket_name()
+                            self.reporter.warning(
+                                "Auto-generated bucket name already taken, retrying with a new name "
+                                f"(attempt {attempt + 1}/{max_attempts}): {bucket_name}"
+                            )
+                            continue
+                        raise
+            else:
+                # User-specified bucket: if not accessible/existing, attempt to create.
+                self.reporter.info(f"Checking TOS bucket accessibility: {bucket_name}")
+                if not tos_service.bucket_exists():
+                    self.reporter.warning(
+                        f"TOS bucket '{bucket_name}' is not accessible or does not exist, attempting to create it..."
+                    )
+                    try:
+                        tos_service.create_bucket()
+                        created_in_this_run = True
+                    except tos.exceptions.TosServerError as e:
+                        if e.status_code == 409:
+                            # The bucket name is already taken (possibly by another account).
+                            # Ownership verification in step 2 will block the upload.
+                            pass
+                        else:
+                            raise
+
+            # Step 2: verify bucket ownership via ListBuckets
+            self.reporter.info(f"Verifying TOS bucket ownership: {bucket_name}")
+
+            def check_owned() -> bool:
+                try:
+                    return tos_service.bucket_is_owned(bucket_name)
+                except Exception as e:
+                    error_msg = (
+                        "Failed to determine TOS bucket ownership via ListBuckets. "
+                        "Upload has been blocked for security reasons. "
+                        "Please ensure your credentials have TOS ListBuckets permission, or set 'tos_bucket: Auto'."
+                    )
                     self.reporter.error(error_msg)
-                    logger.error(error_msg)
+                    logger.error(f"Bucket ownership check failed: {str(e)}")
                     raise Exception(error_msg)
 
-                # Notify user if their configured bucket didn't exist and was auto-created
-                if config.tos_bucket and config.tos_bucket != AUTO_CREATE_VE:
-                    self.reporter.info(
-                        f"Note: Your configured bucket '{config.tos_bucket}' did not exist, auto-created new bucket '{bucket_name}'"
-                    )
+            if created_in_this_run:
+                # ListBuckets may be eventually consistent shortly after creation.
+                timeout_s = 10
+                interval_s = 2
+                deadline = time.time() + timeout_s
+                owned = False
+                while time.time() < deadline:
+                    owned = check_owned()
+                    if owned:
+                        break
+                    time.sleep(interval_s)
             else:
-                self.reporter.success(f"TOS bucket exists: {bucket_name}")
+                owned = check_owned()
+
+            if not owned:
+                error_msg = (
+                    f"Security notice: The configured TOS bucket '{bucket_name}' is not owned by the current account. "
+                    "To prevent uploading your source code to a bucket you do not own (which could leak secrets), this upload has been blocked. "
+                    "Please choose a bucket owned by your account, use 'agentkit config --tos_bucket <your-bucket-name>' to set it."
+                )
+                raise Exception(error_msg)
+
+            self.reporter.success(
+                f"TOS bucket ownership verified for current account: {bucket_name}"
+            )
 
             # Update config with auto-generated bucket name if applicable
             if auto_created_bucket:
