@@ -91,6 +91,22 @@ class VeAgentkitRunnerConfig(AutoSerializableMixin):
         default_factory=dict, metadata={"description": "Runtime environment variables"}
     )
 
+    # Runtime bindings (resource associations)
+    runtime_bindings: Dict[str, Optional[str]] = field(
+        default_factory=dict,
+        metadata={
+            "description": "Runtime associated resources: memory_id/knowledge_id/tool_id/mcp_toolset_id",
+        },
+    )
+
+    # Runtime network configuration (advanced, CreateRuntime only)
+    runtime_network: Dict[str, Any] = field(
+        default_factory=dict,
+        metadata={
+            "description": "Runtime network configuration (advanced, CreateRuntime only)",
+        },
+    )
+
     # Authentication configuration
     runtime_auth_type: str = field(
         default=AUTH_TYPE_KEY_AUTH,
@@ -108,6 +124,14 @@ class VeAgentkitRunnerConfig(AutoSerializableMixin):
     # Container image configuration
     image_url: str = field(default="", metadata={"description": "Container image URL"})
 
+    region: str = field(
+        default="",
+        metadata={
+            "system": True,
+            "description": "AgentKit service region",
+        },
+    )
+
     # Minimum instance count
     min_instance: int = field(
         default=1, metadata={"description": "Minimum number of Runtime instances"}
@@ -140,7 +164,10 @@ class VeAgentkitRuntimeRunner(Runner):
             it does not require a local project directory.
         """
         super().__init__(reporter)
-        self.agentkit_runtime_client = AgentkitRuntimeClient()
+        self.agentkit_runtime_client = None
+
+    def _get_runtime_client(self, region: str = "") -> AgentkitRuntimeClient:
+        return AgentkitRuntimeClient(region=region or "")
 
     def deploy(self, config: VeAgentkitRunnerConfig) -> DeployResult:
         """Deploy Runtime.
@@ -170,7 +197,7 @@ class VeAgentkitRuntimeRunner(Runner):
                 )
 
             # Ensure IAM role exists for Runtime
-            ve_iam = VeIAM()
+            ve_iam = VeIAM(region=config.region)
             if not ve_iam.ensure_role_for_agentkit(runner_config.runtime_role_name):
                 error_msg = "Failed to create or ensure Runtime IAM role."
                 logger.error(error_msg)
@@ -212,7 +239,8 @@ class VeAgentkitRuntimeRunner(Runner):
                 self.reporter.info("Runtime ID not configured, skipping destroy.")
                 return True
 
-            self.agentkit_runtime_client.delete_runtime(
+            client = self._get_runtime_client(config.region)
+            client.delete_runtime(
                 runtime_types.DeleteRuntimeRequest(runtime_id=runner_config.runtime_id)
             )
             self.reporter.success(
@@ -251,7 +279,8 @@ class VeAgentkitRuntimeRunner(Runner):
                     metadata={"message": "Runtime not deployed"},
                 )
 
-            runtime = self.agentkit_runtime_client.get_runtime(
+            client = self._get_runtime_client(config.region)
+            runtime = client.get_runtime(
                 runtime_types.GetRuntimeRequest(runtime_id=runner_config.runtime_id)
             )
 
@@ -410,7 +439,8 @@ class VeAgentkitRuntimeRunner(Runner):
 
                 # Auto-fetch Runtime information if not cached
                 try:
-                    runtime = self.agentkit_runtime_client.get_runtime(
+                    client = self._get_runtime_client(config.region)
+                    runtime = client.get_runtime(
                         runtime_types.GetRuntimeRequest(
                             runtime_id=runner_config.runtime_id
                         )
@@ -437,7 +467,7 @@ class VeAgentkitRuntimeRunner(Runner):
 
                 # Validate based on auth type
                 if not endpoint:
-                    error_msg = "Failed to obtain Runtime endpoint."
+                    error_msg = "Failed to obtain Runtime endpoint. The 'agentkit invoke' command only supports public network endpoints."
                     logger.error(f"{error_msg}, runtime: {runtime}")
                     return InvokeResult(
                         success=False,
@@ -590,6 +620,95 @@ class VeAgentkitRuntimeRunner(Runner):
                 )
             )
 
+    def _build_network_config_for_create(
+        self, config: VeAgentkitRunnerConfig
+    ) -> Optional[runtime_types.NetworkForCreateRuntime]:
+        runtime_network = (
+            config.runtime_network if isinstance(config.runtime_network, dict) else {}
+        )
+        if not runtime_network:
+            return None
+
+        mode = runtime_network.get("mode")
+        vpc_id = runtime_network.get("vpc_id")
+        subnet_ids = runtime_network.get("subnet_ids")
+        enable_shared_internet_access_raw = runtime_network.get(
+            "enable_shared_internet_access"
+        )
+
+        # Convenience: if vpc_id is provided without an explicit mode, assume private.
+        if mode is None and vpc_id:
+            mode = "private"
+
+        if mode is None:
+            raise ValueError(
+                "runtime_network is configured but 'mode' is missing. "
+                "Valid values: public/private/both."
+            )
+
+        mode = str(mode).strip().lower()
+        if mode not in {"public", "private", "both"}:
+            raise ValueError(
+                f"Invalid runtime_network.mode '{mode}'. Valid values: public/private/both."
+            )
+
+        enable_public = mode in {"public", "both"}
+        enable_private = mode in {"private", "both"}
+
+        enable_shared_internet_access: Optional[bool] = None
+        if enable_shared_internet_access_raw is not None:
+            if isinstance(enable_shared_internet_access_raw, bool):
+                enable_shared_internet_access = enable_shared_internet_access_raw
+            elif isinstance(enable_shared_internet_access_raw, (int, float)):
+                enable_shared_internet_access = bool(enable_shared_internet_access_raw)
+            else:
+                raw_str = str(enable_shared_internet_access_raw).strip().lower()
+                if raw_str in {"true", "1", "yes", "y"}:
+                    enable_shared_internet_access = True
+                elif raw_str in {"false", "0", "no", "n"}:
+                    enable_shared_internet_access = False
+                else:
+                    raise ValueError(
+                        "Invalid runtime_network.enable_shared_internet_access. "
+                        "Valid values: true/false."
+                    )
+
+        if enable_shared_internet_access and not enable_private:
+            raise ValueError(
+                "runtime_network.enable_shared_internet_access is only effective when "
+                "runtime_network.mode is private/both."
+            )
+
+        vpc_configuration = None
+        if enable_private:
+            vpc_id_str = str(vpc_id or "").strip()
+            if not vpc_id_str:
+                raise ValueError(
+                    "runtime_network.mode requires 'vpc_id' when mode is private/both."
+                )
+
+            parsed_subnet_ids: Optional[List[str]] = None
+            if isinstance(subnet_ids, str):
+                ids = [s.strip() for s in subnet_ids.split(",") if s.strip()]
+                parsed_subnet_ids = ids or None
+            elif isinstance(subnet_ids, list):
+                ids = [str(s).strip() for s in subnet_ids if str(s).strip()]
+                parsed_subnet_ids = ids or None
+
+            vpc_configuration = runtime_types.NetworkVpcForCreateRuntime(
+                vpc_id=vpc_id_str,
+                subnet_ids=parsed_subnet_ids,
+                enable_shared_internet_access=(
+                    True if enable_shared_internet_access else None
+                ),
+            )
+
+        return runtime_types.NetworkForCreateRuntime(
+            vpc_configuration=vpc_configuration,
+            enable_private_network=enable_private,
+            enable_public_network=enable_public,
+        )
+
     def _create_new_runtime(self, config: VeAgentkitRunnerConfig) -> DeployResult:
         """Create a new Runtime instance.
 
@@ -602,15 +721,30 @@ class VeAgentkitRuntimeRunner(Runner):
         try:
             self.reporter.info(f"Creating Runtime: {config.runtime_name}")
 
+            client = self._get_runtime_client(config.region)
+
             # Build Runtime creation request
             envs = [
                 runtime_types.EnvsItemForCreateRuntime(key=k, value=v)
                 for k, v in config.runtime_envs.items()
             ]
 
+            bindings = (
+                config.runtime_bindings
+                if isinstance(config.runtime_bindings, dict)
+                else {}
+            )
+            memory_id = bindings.get("memory_id")
+            knowledge_id = bindings.get("knowledge_id")
+            tool_id = bindings.get("tool_id")
+            mcp_toolset_id = bindings.get("mcp_toolset_id")
+
             # Build authorizer configuration based on auth type
             authorizer_config = self._build_authorizer_config_for_create(config)
 
+            # Network configuration is only supported during CreateRuntime.
+            network_configuration = self._build_network_config_for_create(config)
+
             create_request = runtime_types.CreateRuntimeRequest(
                 name=config.runtime_name,
                 description=config.common_config.description
@@ -619,6 +753,13 @@ class VeAgentkitRuntimeRunner(Runner):
                 artifact_type=ARTIFACT_TYPE_DOCKER_IMAGE,
                 artifact_url=config.image_url,
                 role_name=config.runtime_role_name,
+                memory_id=(memory_id if is_valid_config(memory_id) else None),
+                knowledge_id=(knowledge_id if is_valid_config(knowledge_id) else None),
+                tool_id=(tool_id if is_valid_config(tool_id) else None),
+                mcp_toolset_id=(
+                    mcp_toolset_id if is_valid_config(mcp_toolset_id) else None
+                ),
+                network_configuration=network_configuration,
                 envs=envs,
                 project_name=PROJECT_NAME_DEFAULT,
                 authorizer_configuration=authorizer_config,
@@ -628,7 +769,7 @@ class VeAgentkitRuntimeRunner(Runner):
             )
 
             # Create Runtime
-            runtime_resp = self.agentkit_runtime_client.create_runtime(create_request)
+            runtime_resp = client.create_runtime(create_request)
             config.runtime_id = runtime_resp.runtime_id
 
             self.reporter.success(f"Runtime created successfully: {config.runtime_id}")
@@ -642,6 +783,7 @@ class VeAgentkitRuntimeRunner(Runner):
                 runtime_id=config.runtime_id,
                 target_status=RUNTIME_STATUS_READY,
                 task_description="Waiting for Runtime to be ready...",
+                region=config.region,
                 timeout=600,
                 error_message="Initialization failed",
             )
@@ -664,7 +806,7 @@ class VeAgentkitRuntimeRunner(Runner):
                         f"Cleaning up failed Runtime: {config.runtime_id}"
                     )
                     try:
-                        self.agentkit_runtime_client.delete_runtime(
+                        client.delete_runtime(
                             runtime_types.DeleteRuntimeRequest(
                                 runtime_id=config.runtime_id
                             )
@@ -781,6 +923,7 @@ class VeAgentkitRuntimeRunner(Runner):
         runtime_id: str,
         target_status: str,
         task_description: str,
+        region: str = "",
         timeout: Optional[int] = None,
         error_message: str = "Failed to wait for Runtime status change",
     ) -> Tuple[bool, Optional[runtime_types.GetRuntimeResponse], Optional[str]]:
@@ -800,6 +943,7 @@ class VeAgentkitRuntimeRunner(Runner):
             runtime_id=runtime_id,
             target_statuses=[target_status],
             task_description=task_description,
+            region=region,
             timeout=timeout,
             error_message=error_message,
         )
@@ -809,6 +953,7 @@ class VeAgentkitRuntimeRunner(Runner):
         runtime_id: str,
         target_statuses: List[str],
         task_description: str,
+        region: str = "",
         timeout: Optional[int] = None,
         error_message: str = "Failed to wait for Runtime status change",
     ) -> Tuple[bool, Optional[runtime_types.GetRuntimeResponse], Optional[str]]:
@@ -833,10 +978,12 @@ class VeAgentkitRuntimeRunner(Runner):
         runtime = None  # Initialize runtime variable
 
         # Use reporter.long_task() for progress tracking
+        client = self._get_runtime_client(region)
+
         with self.reporter.long_task(task_description, total=total_time) as task:
             while True:
                 runtime = retry(
-                    lambda: self.agentkit_runtime_client.get_runtime(
+                    lambda: client.get_runtime(
                         runtime_types.GetRuntimeRequest(runtime_id=runtime_id)
                     )
                 )
@@ -963,9 +1110,17 @@ class VeAgentkitRuntimeRunner(Runner):
         try:
             self.reporter.info(f"Updating Runtime: {config.runtime_id}")
 
+            if isinstance(config.runtime_network, dict) and config.runtime_network:
+                self.reporter.warning(
+                    "runtime_network is configured, but network settings only apply when creating a Runtime. "
+                    "UpdateRuntime does not support network_configuration; ignoring runtime_network."
+                )
+
+            client = self._get_runtime_client(config.region)
+
             # Get existing Runtime information
             try:
-                runtime = self.agentkit_runtime_client.get_runtime(
+                runtime = client.get_runtime(
                     runtime_types.GetRuntimeRequest(runtime_id=config.runtime_id)
                 )
             except Exception as e:
@@ -1024,14 +1179,44 @@ class VeAgentkitRuntimeRunner(Runner):
 
             self.reporter.info("Starting Runtime update...")
 
+            def _binding_update_value(key: str) -> Optional[str]:
+                """Translate runtime_bindings into UpdateRuntimeRequest fields.
+
+                Semantics:
+                - key not present: return None (do not send)
+                - value is None: return "" (explicit clear/unbind)
+                - value is "" or whitespace: return "" (explicit clear/unbind)
+                - value is non-empty: return value
+                """
+                if (
+                    not isinstance(config.runtime_bindings, dict)
+                    or key not in config.runtime_bindings
+                ):
+                    return None
+                raw = config.runtime_bindings.get(key)
+                if raw is None:
+                    return ""
+                if isinstance(raw, str) and not raw.strip():
+                    return ""
+                return str(raw)
+
+            memory_id = _binding_update_value("memory_id")
+            knowledge_id = _binding_update_value("knowledge_id")
+            tool_id = _binding_update_value("tool_id")
+            mcp_toolset_id = _binding_update_value("mcp_toolset_id")
+
             envs = [
                 {"Key": str(k), "Value": str(v)} for k, v in config.runtime_envs.items()
             ]
-            self.agentkit_runtime_client.update_runtime(
+            client.update_runtime(
                 runtime_types.UpdateRuntimeRequest(
                     runtime_id=config.runtime_id,
                     artifact_url=config.image_url,
                     description=config.common_config.description,
+                    memory_id=memory_id,
+                    knowledge_id=knowledge_id,
+                    tool_id=tool_id,
+                    mcp_toolset_id=mcp_toolset_id,
                     envs=envs,
                     client_token=generate_client_token(),
                 )
@@ -1045,6 +1230,7 @@ class VeAgentkitRuntimeRunner(Runner):
                 runtime_id=config.runtime_id,
                 target_statuses=[RUNTIME_STATUS_UNRELEASED, RUNTIME_STATUS_READY],
                 task_description="Waiting for Runtime update to complete...",
+                region=config.region,
                 timeout=600,
                 error_message="Update failed",
             )
@@ -1071,7 +1257,7 @@ class VeAgentkitRuntimeRunner(Runner):
             else:
                 # Phase 2: Status is UnReleased, need to release the update
                 self.reporter.info("Starting Runtime release...")
-                self.agentkit_runtime_client.release_runtime(
+                client.release_runtime(
                     runtime_types.ReleaseRuntimeRequest(
                         runtime_id=config.runtime_id,
                     )
@@ -1089,6 +1275,7 @@ class VeAgentkitRuntimeRunner(Runner):
                     runtime_id=config.runtime_id,
                     target_status=RUNTIME_STATUS_READY,
                     task_description="Waiting for Runtime release to complete...",
+                    region=config.region,
                     timeout=300,
                     error_message="Release failed",
                 )
